window.Gleam.initSearch([{"doc":"shine_tree","title":"shine_tree","content":"# shine_tree\n\n[![Package Version](https://img.shields.io/hexpm/v/shine_tree)](https://hex.pm/packages/shine_tree)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/shine_tree/)\n\nA `FingerTree` implementation for general purpose functional programming in Gleam!\n\n```rs\npub opaque type ShineTree(u) {\n  Empty\n  Single(u)\n  Deep(count, Node(u), ShineTree(Node(u)), Node(u))\n}\n\ntype Node(u) {\n  One(u)\n  Two(u, u)\n  Three(u, u, u)\n  Four(u, u, u, u)\n}\n```\n\n## Usage\n\nTo install shine_tree into your project, use the following command.\n\n\n```sh\ngleam add shine_tree\n```\n\n```gleam\nimport shine_tree.{empty, push, to_list}\n\npub fn main() {\n  empty\n  |&gt; push(1)\n  |&gt; push(2)\n  |&gt; push(3)\n  |&gt; to_list\n  |&gt; io.debug\n  // [1, 2, 3]\n}\n```\n\nMost of the list functions in the std library have been implemented along with many more convenience methods like:\n\n- push\n- pop\n- shift\n- unshift\n- from_list\n- to_list\n- from_iterator\n- TODO: to_iterator\n- TODO: to_set\n- TODO: sort\n\nFurther documentation can be found at &lt;https://hexdocs.pm/shine_tree&gt;.\n\n## License\n\n```\nCopyright © 2024 Joshua Tenner &lt;tenner.joshua@gmail.com&gt;\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n```\n","url":"index.html"},{"doc":"shine_tree","title":"shine_tree","content":"# ShineTree Benchmarks\n\n## append\n\n   Compiled in 0.05s\n    Running benchmarks/append.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|list.append -&gt; n = 100|0.45us|0.0us|43.46|0.0us|45.37ms|8008228|\n|list.append -&gt; n = 1_000|3.88us|2.0us|45.63|2.0us|26.98ms|1251932|\n|list.append -&gt; n = 10_000|26.33us|21.0us|73.68|19.0us|20.29ms|188810|\n|list.append -&gt; n = 100_000|413.15us|374.0us|509.95|167.0us|30.3ms|12128|\n|shine_tree.append -&gt; n = 100|0.38us|0.0us|92.8|0.0us|98.85ms|10053552|\n|shine_tree.append -&gt; n = 1_000|0.69us|0.0us|102.17|0.0us|123.6ms|6287248|\n|shine_tree.append -&gt; n = 10_000|0.91us|0.0us|102.63|0.0us|85.49ms|4888296|\n|shine_tree.append -&gt; n = 100_000|1.13us|1.0us|97.72|0.0us|43.16ms|4046049|\n\n\n\n## count\n\n   Compiled in 0.05s\n    Running benchmarks/count.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|List count evens -&gt; n = 100|1.74us|1.0us|17.5|1.0us|16.11ms|2659214|\n|List count evens -&gt; n = 1_000|16.89us|15.0us|63.76|15.0us|20.05ms|293807|\n|List count evens -&gt; n = 10_000|168.63us|158.0us|186.4|149.0us|27.42ms|29609|\n|List count evens -&gt; n = 100_000|1.91ms|1.57ms|1507.77|1.5ms|33.23ms|2621|\n|ShineTree count evens -&gt; n = 100|1.39us|2.0us|31.95|1.0us|31.92ms|3346790|\n|ShineTree count evens -&gt; n = 1_000|13.41us|12.0us|73.91|11.0us|22.92ms|370217|\n|ShineTree count evens -&gt; n = 10_000|133.56us|124.0us|175.53|121.0us|24.06ms|37387|\n|ShineTree count evens -&gt; n = 100_000|1.32ms|1.22ms|497.29|1.2ms|14.03ms|3778|\n\n\n\n## from_iterator\n\n   Compiled in 0.05s\n    Running benchmarks/from_iterator.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|iterator.to_list -&gt; n = 100|5.4us|3.0us|121.06|2.0us|64.36ms|902626|\n|iterator.to_list -&gt; n = 1_000|36.99us|55.0us|130.39|26.0us|20.14ms|134752|\n|iterator.to_list -&gt; n = 10_000|593.1us|572.0us|641.0|374.0us|35.43ms|8425|\n|iterator.to_list -&gt; n = 100_000|6.42ms|5.85ms|1201.07|5.3ms|18.91ms|778|\n|shine_tree.from_iterator -&gt; n = 100|9.65us|4.0us|114.16|3.0us|37.67ms|512812|\n|shine_tree.from_iterator -&gt; n = 1_000|86.01us|45.0us|238.47|40.0us|26.28ms|58022|\n|shine_tree.from_iterator -&gt; n = 10_000|699.59us|526.0us|332.07|485.0us|15.16ms|7145|\n|shine_tree.from_iterator -&gt; n = 100_000|7.37ms|6.78ms|2033.91|6.48ms|43.26ms|678|\n\n\n\n## shift\n\n   Compiled in 0.04s\n    Running benchmarks/shift.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|list (take one item at a time, until empty) -&gt; n = 100|0.29us|0.0us|4.93|0.0us|9.94ms|11425501|\n|list (take one item at a time, until empty) -&gt; n = 1_000|2.89us|3.0us|29.12|2.0us|29.71ms|1641178|\n|list (take one item at a time, until empty) -&gt; n = 10_000|26.25us|25.0us|13.87|23.0us|3.35ms|189367|\n|list (take one item at a time, until empty) -&gt; n = 100_000|253.75us|289.0us|207.52|228.0us|22.06ms|19685|\n|shine_tree.shift (against best case scenario list) -&gt; n = 100|7.32us|2.0us|116.99|2.0us|31.27ms|671737|\n|shine_tree.shift (against best case scenario list) -&gt; n = 1_000|71.13us|30.0us|197.35|24.0us|9.67ms|70104|\n|shine_tree.shift (against best case scenario list) -&gt; n = 10_000|705.47us|298.0us|543.93|291.0us|29.89ms|7085|\n|shine_tree.shift (against best case scenario list) -&gt; n = 100_000|8.48ms|7.49ms|2388.11|6.86ms|47.71ms|590|\n|shine_tree.pop (lists can&#39;t do this at all) -&gt; n = 100|7.19us|4.0us|119.1|1.0us|35.57ms|684529|\n|shine_tree.pop (lists can&#39;t do this at all) -&gt; n = 1_000|70.41us|27.0us|230.1|21.0us|29.65ms|70843|\n|shine_tree.pop (lists can&#39;t do this at all) -&gt; n = 10_000|679.55us|856.0us|525.0|267.0us|25.62ms|7354|\n|shine_tree.pop (lists can&#39;t do this at all) -&gt; n = 100_000|8.05ms|7.64ms|1694.46|6.6ms|33.13ms|621|\n\n\n\n## sort\n\n   Compiled in 0.05s\n    Running benchmarks/sort.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|List sort -&gt; n = 100|10.32us|7.0us|76.39|6.0us|26.59ms|479416|\n|List sort -&gt; n = 1_000|150.95us|137.0us|240.94|128.0us|39.59ms|33091|\n|List sort -&gt; n = 10_000|2.19ms|2.13ms|314.77|2.05ms|11.8ms|2283|\n|List sort -&gt; n = 100_000|34.54ms|32.0ms|5845.32|29.02ms|73.15ms|145|\n|ShineTree sort -&gt; n = 100|13.26us|8.0us|114.05|7.0us|28.83ms|373556|\n|ShineTree sort -&gt; n = 1_000|212.22us|147.0us|305.31|136.0us|19.37ms|23539|\n|ShineTree sort -&gt; n = 10_000|2.78ms|2.61ms|929.75|1.83ms|21.18ms|1795|\n|ShineTree sort -&gt; n = 100_000|34.3ms|36.0ms|5639.34|27.98ms|70.42ms|145|\n\n\n\n## fold\n\n   Compiled in 0.05s\n    Running benchmarks/fold.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|list.fold -&gt; n = 100|0.94us|1.0us|22.86|0.0us|17.44ms|4433277|\n|list.fold -&gt; n = 1_000|8.63us|7.0us|57.69|7.0us|31.15ms|570367|\n|list.fold -&gt; n = 10_000|87.65us|81.0us|164.64|78.0us|22.0ms|56930|\n|list.fold -&gt; n = 100_000|876.99us|846.0us|445.98|785.0us|23.9ms|5699|\n|iterator.fold -&gt; n = 100|4.96us|3.0us|91.63|2.0us|21.32ms|989189|\n|iterator.fold -&gt; n = 1_000|40.72us|24.0us|153.38|21.0us|32.53ms|122427|\n|iterator.fold -&gt; n = 10_000|234.14us|217.0us|138.77|216.0us|14.99ms|21340|\n|iterator.fold -&gt; n = 100_000|2.4ms|2.67ms|768.9|2.22ms|25.24ms|2081|\n|shine_tree.fold_l -&gt; n = 100|0.57us|0.0us|46.35|0.0us|73.13ms|7409110|\n|shine_tree.fold_l -&gt; n = 1_000|5.47us|5.0us|45.07|4.0us|33.64ms|897499|\n|shine_tree.fold_l -&gt; n = 10_000|56.67us|48.0us|150.68|46.0us|29.0ms|88000|\n|shine_tree.fold_l -&gt; n = 100_000|577.29us|584.0us|281.3|467.0us|15.68ms|8656|\n\n\n\n## range\n\n   Compiled in 0.07s\n    Running benchmarks/range.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|List range of size n -&gt; n = 100|1.46us|1.0us|75.37|0.0us|32.18ms|3172242|\n|List range of size n -&gt; n = 1_000|13.45us|8.0us|91.2|7.0us|9.7ms|368792|\n|List range of size n -&gt; n = 10_000|104.53us|95.0us|185.06|79.0us|22.47ms|47762|\n|List range of size n -&gt; n = 100_000|1.24ms|1.32ms|702.54|1.02ms|24.76ms|4016|\n|ShineTree range of size n -&gt; n = 100|1.05us|1.0us|89.92|0.0us|49.76ms|4359871|\n|ShineTree range of size n -&gt; n = 1_000|8.73us|5.0us|106.94|3.0us|29.09ms|565988|\n|ShineTree range of size n -&gt; n = 10_000|74.34us|45.0us|165.46|39.0us|18.3ms|67127|\n|ShineTree range of size n -&gt; n = 100_000|2.62ms|2.1ms|954.76|1.8ms|24.82ms|1911|\n\n\n\n## size\n\n   Compiled in 0.04s\n    Running benchmarks/size.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|list.size -&gt; n = 100|0.16us|0.0us|10.0|0.0us|33.55ms|15711838|\n|list.size -&gt; n = 1_000|2.22us|2.0us|24.19|1.0us|18.43ms|2092006|\n|list.size -&gt; n = 10_000|16.47us|33.0us|47.71|14.0us|21.14ms|300046|\n|list.size -&gt; n = 100_000|113.46us|105.0us|48.65|102.0us|7.32ms|43993|\n|shine_tree.size -&gt; n = 100|0.03us|0.0us|6.54|0.0us|32.4ms|28133030|\n|shine_tree.size -&gt; n = 1_000|0.03us|0.0us|3.54|0.0us|10.47ms|20487610|\n|shine_tree.size -&gt; n = 10_000|0.04us|0.0us|7.67|0.0us|29.44ms|20185780|\n|shine_tree.size -&gt; n = 100_000|0.03us|0.0us|6.33|0.0us|19.17ms|22538207|\n\n\n\n## unshift\n\n   Compiled in 0.05s\n    Running benchmarks/unshift.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|shine_tree.unshift(Nil) -&gt; n = 100|6.28us|2.0us|133.67|1.0us|35.23ms|779441|\n|shine_tree.unshift(Nil) -&gt; n = 1_000|55.73us|23.0us|188.99|16.0us|24.72ms|89388|\n|shine_tree.unshift(Nil) -&gt; n = 10_000|535.85us|294.0us|562.53|244.0us|39.83ms|9325|\n|shine_tree.unshift(Nil) -&gt; n = 100_000|6.56ms|6.47ms|1978.9|3.95ms|35.21ms|762|\n|[item, ..rest] -&gt; n = 100|1.02us|0.0us|125.73|0.0us|64.91ms|4207056|\n|[item, ..rest] -&gt; n = 1_000|8.87us|3.0us|152.06|1.0us|35.17ms|552812|\n|[item, ..rest] -&gt; n = 10_000|71.26us|41.0us|277.62|21.0us|29.46ms|69919|\n|[item, ..rest] -&gt; n = 100_000|985.82us|590.0us|859.53|447.0us|35.85ms|5065|\n|shine_tree.push(Nil) (lists can&#39;t do this at all) -&gt; n = 100|6.07us|4.0us|140.58|1.0us|32.57ms|809162|\n|shine_tree.push(Nil) (lists can&#39;t do this at all) -&gt; n = 1_000|62.99us|1.19ms|290.01|13.0us|48.32ms|79192|\n|shine_tree.push(Nil) (lists can&#39;t do this at all) -&gt; n = 10_000|539.36us|337.0us|443.14|214.0us|14.85ms|9265|\n|shine_tree.push(Nil) (lists can&#39;t do this at all) -&gt; n = 100_000|7.44ms|6.22ms|3020.44|3.73ms|43.14ms|672|\n\n\n\n## group\n\n   Compiled in 0.06s\n    Running benchmarks/group.main\n\n| Name | Mean | Median | Std. | Min | Max | Iterations |\n|---|---|---|---|---|---|---|\n|list.group (n % 7) -&gt; n = 100|11.25us|8.0us|105.91|7.0us|44.01ms|438551|\n|list.group (n % 7) -&gt; n = 1_000|113.07us|86.0us|224.34|75.0us|30.15ms|44115|\n|list.group (n % 7) -&gt; n = 10_000|1.31ms|1.31ms|779.38|871.0us|31.09ms|3813|\n|list.group (n % 7) -&gt; n = 100_000|13.35ms|15.47ms|3435.15|9.37ms|41.01ms|373|\n|iterator.group (n % 7) -&gt; n = 100|14.49us|9.0us|31.2|8.0us|6.19ms|341775|\n|iterator.group (n % 7) -&gt; n = 1_000|129.84us|98.0us|254.3|89.0us|30.4ms|38450|\n|iterator.group (n % 7) -&gt; n = 10_000|1.31ms|1.2ms|578.28|1.01ms|21.3ms|3804|\n|iterator.group (n % 7) -&gt; n = 100_000|17.53ms|17.36ms|4425.71|14.09ms|71.34ms|285|\n|shine_tree.group (n % 7) -&gt; n = 100|9.09us|6.0us|96.96|5.0us|11.48ms|541024|\n|shine_tree.group (n % 7) -&gt; n = 1_000|105.09us|72.0us|236.11|62.0us|20.16ms|47397|\n|shine_tree.group (n % 7) -&gt; n = 10_000|1.09ms|715.0us|855.06|673.0us|21.11ms|4595|\n|shine_tree.group (n % 7) -&gt; n = 100_000|10.71ms|16.49ms|2262.55|7.38ms|20.63ms|467|\n\n\n\n","url":"benchmarks.html"},{"doc":"shine_tree","title":"shine_tree","content":"# Examples\n\nA simple FIFO work queue:\n\n```gleam\npub type Queue(item) {\n  Queue(items: FingerTree(item))\n}\n\npub fn new() {\n  shine_tree.empty\n}\n\npub fn enqueue(queue: Queue(item), item) {\n  Queue(shine_tree.push(queue.items, item))\n}\n\npub fn dequeue(queue: Queue(item)) {\n  case shine_tree.shift(queue.items) {\n    Error(_) -&gt; None\n    Ok(#(item, rest)) -&gt; Some(#(item, Queue(rest)))\n  }\n}\n","url":"examples.html"},{"doc":"shine_tree","title":"ShineTree","content":"pub opaque type ShineTree(u)\n An ordered tree of values used to implement other kinds of data\n structures like queues. Elements can be added or removed from the\n head or tail of the tree in nearly constant time.\n\nSynonyms:\nshine_tree.ShineTree\nshine_tree ShineTree","url":"shine_tree.html#ShineTree"},{"doc":"shine_tree","title":"empty","content":"pub const empty: ShineTree(a)\n\nSynonyms:\nshine_tree.empty\nshine_tree empty","url":"shine_tree.html#empty"},{"doc":"shine_tree","title":"fold","content":"pub const fold: fn(ShineTree(a), b, fn(c, a) -&gt; c) -&gt; c\n This is an alias for `fold_l`.\nSynonyms:\nshine_tree.fold\nshine_tree fold","url":"shine_tree.html#fold"},{"doc":"shine_tree","title":"fold_right","content":"pub const fold_right: fn(ShineTree(a), b, fn(c, a) -&gt; c) -&gt; c\n This is an alias for `fold_r`.\nSynonyms:\nshine_tree.fold_right\nshine_tree fold_right","url":"shine_tree.html#fold_right"},{"doc":"shine_tree","title":"fold_right_until","content":"pub const fold_right_until: fn(\n  ShineTree(a),\n  b,\n  fn(b, a) -&gt; ContinueOrStop(b),\n) -&gt; b\n This is an alias for `fold_r_until`\nSynonyms:\nshine_tree.fold_right_until\nshine_tree fold_right_until","url":"shine_tree.html#fold_right_until"},{"doc":"shine_tree","title":"fold_until","content":"pub const fold_until: fn(\n  ShineTree(a),\n  b,\n  fn(b, a) -&gt; ContinueOrStop(b),\n) -&gt; b\n This is an alias for `fold_l_until`\nSynonyms:\nshine_tree.fold_until\nshine_tree fold_until","url":"shine_tree.html#fold_until"},{"doc":"shine_tree","title":"head","content":"pub const head: fn(ShineTree(a)) -&gt; Result(a, Nil)\n This is an alias for `first`.\nSynonyms:\nshine_tree.head\nshine_tree head","url":"shine_tree.html#head"},{"doc":"shine_tree","title":"tail","content":"pub const tail: fn(ShineTree(a)) -&gt; Result(a, Nil)\n This is an alias for `last`.\nSynonyms:\nshine_tree.tail\nshine_tree tail","url":"shine_tree.html#tail"},{"doc":"shine_tree","title":"try_fold","content":"pub const try_fold: fn(ShineTree(a), b, fn(b, a) -&gt; Result(b, c)) -&gt;\n  Result(b, c)\n This is an alias for `try_foldl`\nSynonyms:\nshine_tree.try_fold\nshine_tree try_fold","url":"shine_tree.html#try_fold"},{"doc":"shine_tree","title":"all","content":"pub fn all(tree: ShineTree(a), f: fn(a) -&gt; Bool) -&gt; Bool\n Returns `true` if the predicate `f` returns `True` for each\n element in the ShineTree. Returns `False` otherwise.\n \n ## Examples\n\n \n shine_tree.all(shine_tree.empty, fn(x) { x == 0 })\n // -&gt; True\n \n\n \n all(shine_tree.from_list([-1, -42]), fn(x) { x &lt; 0 })\n // -&gt; True\n \n\n \n all(shine_tree.from_list([2, 4, 6, 8, 10, 11]), int.is_even)\n // -&gt; False\n \nSynonyms:\nshine_tree.all\nshine_tree all","url":"shine_tree.html#all"},{"doc":"shine_tree","title":"any","content":"pub fn any(tree: ShineTree(a), f: fn(a) -&gt; Bool) -&gt; Bool\n Returns `True` if the predicate `f` returns `True` for any\n element in the ShineTree. Returns `False` otherwise.\n \n ## Examples\n\n \n // No items in the tree will always be false\n any(shine_tree.empty, int.is_even)\n // -&gt; False\n \n\n \n any([10, 32, 42, 43, 44], fn(x) { x == 42 })\n // -&gt; True\n \n \n \n any([3, 4], fn(x) { x + 1 == 42 })\n // -&gt; False\n \nSynonyms:\nshine_tree.any\nshine_tree any","url":"shine_tree.html#any"},{"doc":"shine_tree","title":"append","content":"pub fn append(\n  tree_1: ShineTree(a),\n  tree_2: ShineTree(a),\n) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.append\nshine_tree append","url":"shine_tree.html#append"},{"doc":"shine_tree","title":"concat","content":"pub fn concat(trees: List(ShineTree(a))) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.concat\nshine_tree concat","url":"shine_tree.html#concat"},{"doc":"shine_tree","title":"contains","content":"pub fn contains(tree: ShineTree(a), u: a) -&gt; Bool\n Returns `True` if the given value is contained in the given tree.\nSynonyms:\nshine_tree.contains\nshine_tree contains","url":"shine_tree.html#contains"},{"doc":"shine_tree","title":"count","content":"pub fn count(tree: ShineTree(a), where f: fn(a) -&gt; Bool) -&gt; Int\n Counts the number of elements in a given ShineTree satisfying a given predicate.\n\n This function has to traverse the entire tree to determine the number of elements,\n so it runs in linear time.\n \n ## Examples\n \n \n count(shine_tree.from_list([2, 3, 4, 5, 6, 7, 8, 9, 10]), int.is_even)\n // -&gt; 5\n \nSynonyms:\nshine_tree.count\nshine_tree count","url":"shine_tree.html#count"},{"doc":"shine_tree","title":"each","content":"pub fn each(tree: ShineTree(a), f: fn(a) -&gt; b) -&gt; Nil\n Calls a function for each element in the tree, discarding the return value.\nSynonyms:\nshine_tree.each\nshine_tree each","url":"shine_tree.html#each"},{"doc":"shine_tree","title":"equals","content":"pub fn equals(a: ShineTree(a), b: ShineTree(a)) -&gt; Bool\n Compares two trees for equality. Since trees can have many shapes, the items\n can be in the correct order and values, but the tree is balanced differently.\n \n This function reduces over the two trees and compares their values, immediately\n returning `False` if they are not equal.\n \nSynonyms:\nshine_tree.equals\nshine_tree equals","url":"shine_tree.html#equals"},{"doc":"shine_tree","title":"filter","content":"pub fn filter(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; Bool,\n) -&gt; ShineTree(a)\n Creates a new tree containing all the elements of the given tree,\n for which the given predicate returns `True`.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; filter(int.is_odd)\n // -&gt; shine_tree.from_list([1, 3, 5, 7])\n \n \nSynonyms:\nshine_tree.filter\nshine_tree filter","url":"shine_tree.html#filter"},{"doc":"shine_tree","title":"filter_map","content":"pub fn filter_map(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; Result(b, c),\n) -&gt; ShineTree(b)\n Returns a new tree containing only the elements from the first tree\n where the given function returns `Ok(b)`.\nSynonyms:\nshine_tree.filter_map\nshine_tree filter_map","url":"shine_tree.html#filter_map"},{"doc":"shine_tree","title":"find","content":"pub fn find(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; Bool,\n) -&gt; Result(a, Nil)\n Finds the first element in a given tree that satisfies the given predicate.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; find(int.is_even)\n // -&gt; 2\n \n \nSynonyms:\nshine_tree.find\nshine_tree find","url":"shine_tree.html#find"},{"doc":"shine_tree","title":"find_map","content":"pub fn find_map(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; Result(b, c),\n) -&gt; Result(b, Nil)\n Finds the first element in a given tree for which the given function returns `Ok(val)`.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; find_map(fn(u) {\n   case u &gt; 5 {\n     True -&gt; Ok(u)\n     False -&gt; Error(Nil)\n   }\n })\n // -&gt; 2\n \n \nSynonyms:\nshine_tree.find_map\nshine_tree find_map","url":"shine_tree.html#find_map"},{"doc":"shine_tree","title":"first","content":"pub fn first(tree: ShineTree(a)) -&gt; Result(a, Nil)\n\nSynonyms:\nshine_tree.first\nshine_tree first","url":"shine_tree.html#first"},{"doc":"shine_tree","title":"fold_l","content":"pub fn fold_l(\n  over tree: ShineTree(a),\n  from v: b,\n  with f: fn(c, a) -&gt; c,\n) -&gt; c\n Reduces all the elements of the given tree into a single value,\n made by calling a given function on all the elements, traversing\n the tree from left to right.\n\n `fold_l(shine_tree.from_list([1, 2, 3]), 0, int.add)` is the equivalent of\n `add(add(add(0, 1), 2), 3)`.\n\n \n // calculate 20! (factorial)\n let n = 20\n shine_tree.fold_l(shine_tree.range(2, n), int.multiply)\n // -&gt; 2432902008176640000\n \nSynonyms:\nshine_tree.fold_l\nshine_tree fold_l","url":"shine_tree.html#fold_l"},{"doc":"shine_tree","title":"fold_l_until","content":"pub fn fold_l_until(\n  tree: ShineTree(a),\n  v: b,\n  with f: fn(b, a) -&gt; ContinueOrStop(b),\n) -&gt; b\n Fold a given value over the items of a given tree, starting with the beginning until the\n given function returns `Stop`.\n \n The accumulated value is then returned.\n \n ## Examples\n \n \n fold_until(shine_tree.from_list([2, 3, 4, 5, 6, 7, 8, 9, 10]), 0, fn(acc, x) {\n   case x {\n     5 -&gt; Stop(acc + 1)\n     _ -&gt; Continue(acc + x)\n   }\n })\n // -&gt; 10\nSynonyms:\nshine_tree.fold_l_until\nshine_tree fold_l_until","url":"shine_tree.html#fold_l_until"},{"doc":"shine_tree","title":"fold_r","content":"pub fn fold_r(tree: ShineTree(a), v: b, f: fn(c, a) -&gt; c) -&gt; c\n Reduces all the elements of the given tree into a single value,\n made by calling a given function on all the elements, traversing\n the tree from left to right.\n\n `fold_r(shine_tree.from_list([1, 2, 3]), 0, int.add)` is the equivalent of\n `add(add(add(0, 3), 2), 1)`.\n\n \n // calculate 20! (factorial)\n let n = 20\n shine_tree.fold_r(shine_tree.range(2, n), int.multiply)\n // -&gt; 2432902008176640000\n \nSynonyms:\nshine_tree.fold_r\nshine_tree fold_r","url":"shine_tree.html#fold_r"},{"doc":"shine_tree","title":"fold_r_until","content":"pub fn fold_r_until(\n  tree: ShineTree(a),\n  v: b,\n  with f: fn(b, a) -&gt; ContinueOrStop(b),\n) -&gt; b\n Fold a given value over the items of a given tree, starting with the end until the\n given function returns `Stop`.\n \n The accumulated value is then returned.\nSynonyms:\nshine_tree.fold_r_until\nshine_tree fold_r_until","url":"shine_tree.html#fold_r_until"},{"doc":"shine_tree","title":"from_iterator","content":"pub fn from_iterator(iterable: Iterator(a)) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.from_iterator\nshine_tree from_iterator","url":"shine_tree.html#from_iterator"},{"doc":"shine_tree","title":"from_list","content":"pub fn from_list(values: List(a)) -&gt; ShineTree(a)\n Creates a new tree containing all the elements of the given list.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n // -&gt; It&#39;s a ShineTree with all the items in it!\n // Were you expecting a list?\n \n \nSynonyms:\nshine_tree.from_list\nshine_tree from_list","url":"shine_tree.html#from_list"},{"doc":"shine_tree","title":"get","content":"pub fn get(tree: ShineTree(a), index: Int) -&gt; Result(a, Nil)\n Get the value at the given index in the tree if it exists.\nSynonyms:\nshine_tree.get\nshine_tree get","url":"shine_tree.html#get"},{"doc":"shine_tree","title":"group","content":"pub fn group(\n  tree: ShineTree(a),\n  f: fn(a) -&gt; b,\n) -&gt; Dict(b, ShineTree(a))\n This function takes the tree, and groups the values by a returned key into a `Dict`.\n It does not sort the values, or preserve the order.\nSynonyms:\nshine_tree.group\nshine_tree group","url":"shine_tree.html#group"},{"doc":"shine_tree","title":"last","content":"pub fn last(tree: ShineTree(a)) -&gt; Result(a, Nil)\n\nSynonyms:\nshine_tree.last\nshine_tree last","url":"shine_tree.html#last"},{"doc":"shine_tree","title":"map","content":"pub fn map(\n  tree: ShineTree(a),\n  with f: fn(a) -&gt; b,\n) -&gt; ShineTree(b)\n Maps all the elements of the given tree into a new tree where\n each element has been transformed by the given function.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.map(tree, int.multiply(2, _))\n // -&gt; shine_tree.from_list([2, 4, 6])\n \nSynonyms:\nshine_tree.map\nshine_tree map","url":"shine_tree.html#map"},{"doc":"shine_tree","title":"pop","content":"pub fn pop(tree: ShineTree(a)) -&gt; Result(#(a, ShineTree(a)), Nil)\n Pops an element from the end of the given tree.\n \n If there are no items, it returns `Error(Nil)`, otherwise\n it returns the popped item, and the resulting tree with the\n element removed.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.pop(tree)\n // -&gt; Ok(3, shine_tree.from_list([1, 2]))\n \n \n \n shine_tree.empty |&gt; shine_tree.pop\n // -&gt; Error(Nil)\n \nSynonyms:\nshine_tree.pop\nshine_tree pop","url":"shine_tree.html#pop"},{"doc":"shine_tree","title":"prepend","content":"pub fn prepend(\n  tree_1: ShineTree(a),\n  tree_2: ShineTree(a),\n) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.prepend\nshine_tree prepend","url":"shine_tree.html#prepend"},{"doc":"shine_tree","title":"push","content":"pub fn push(tree: ShineTree(a), value: a) -&gt; ShineTree(a)\n Pushes an element to the end of the given tree.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.push(tree, 4)\n // -&gt; shine_tree.from_list([1, 2, 3, 4])\n \nSynonyms:\nshine_tree.push\nshine_tree push","url":"shine_tree.html#push"},{"doc":"shine_tree","title":"range","content":"pub fn range(start: Int, finish: Int) -&gt; ShineTree(Int)\n Creates a tree of `n` consecutive integers, starting from `start`.\n and finishing with `finish`.\n \n ## Examples\n \n \n let ten_items = shine_tree.range(1, 10)\n \n \nSynonyms:\nshine_tree.range\nshine_tree range","url":"shine_tree.html#range"},{"doc":"shine_tree","title":"reverse","content":"pub fn reverse(tree: ShineTree(a)) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.reverse\nshine_tree reverse","url":"shine_tree.html#reverse"},{"doc":"shine_tree","title":"shift","content":"pub fn shift(\n  tree: ShineTree(a),\n) -&gt; Result(#(a, ShineTree(a)), Nil)\n Pops an element from the front of the given tree.\n \n If there are no items, it returns `Error(Nil)`, otherwise\n it returns the popped item, and the resulting tree with the\n element removed.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.shift(tree)\n // -&gt; Ok(1, shine_tree.from_list([2, 3]))\n \n \n \n shine_tree.empty |&gt; shine_tree.pop\n // -&gt; Error(Nil)\n \nSynonyms:\nshine_tree.shift\nshine_tree shift","url":"shine_tree.html#shift"},{"doc":"shine_tree","title":"single","content":"pub fn single(u: a) -&gt; ShineTree(a)\n\nSynonyms:\nshine_tree.single\nshine_tree single","url":"shine_tree.html#single"},{"doc":"shine_tree","title":"size","content":"pub fn size(tree: ShineTree(a)) -&gt; Int\n Returns the number of elements in the given tree.\n \n This is an O(1) operation, because the size is cached in the tree\n at the time it is created.\nSynonyms:\nshine_tree.size\nshine_tree size","url":"shine_tree.html#size"},{"doc":"shine_tree","title":"sort","content":"pub fn sort(\n  tree: ShineTree(a),\n  compare: fn(a, a) -&gt; Order,\n) -&gt; ShineTree(a)\n Sort a `ShineTree` using the `compare` function using a &quot;quicksort&quot; algorithm.\nSynonyms:\nshine_tree.sort\nshine_tree sort","url":"shine_tree.html#sort"},{"doc":"shine_tree","title":"to_iterator","content":"pub fn to_iterator(tree: ShineTree(a)) -&gt; Iterator(a)\n Creates an iterator from a given tree, yielding each element in succession until\n there are no elements left.\n\n ## Examples\n\n \n let iter = unfold(shine_tree.from_list([1, 2, 3]))\n \n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(1, Iterator(Int))\n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(2, Iterator(Int))\n let #(num, iter) = iterator.step(iter)\n // -&gt; Next(3, Iterator(Int))\n let a = iterator.step(iter)\n // -&gt; Done\n \n\nSynonyms:\nshine_tree.to_iterator\nshine_tree to_iterator","url":"shine_tree.html#to_iterator"},{"doc":"shine_tree","title":"to_list","content":"pub fn to_list(tree: ShineTree(a)) -&gt; List(a)\n Creates a new list containing all the elements of the given tree.\n \n \n shine_tree.from_list([1, 2, 3, 4, 5, 6, 7])\n |&gt; to_list\n // -&gt; [1, 2, 3, 4, 5, 6, 7]\n \n \nSynonyms:\nshine_tree.to_list\nshine_tree to_list","url":"shine_tree.html#to_list"},{"doc":"shine_tree","title":"try_foldl","content":"pub fn try_foldl(\n  tree: ShineTree(a),\n  acc: b,\n  f: fn(b, a) -&gt; Result(b, c),\n) -&gt; Result(b, c)\n Perform a folding operation and return the success `Ok` value,\n or the error `Error` value if the operation fails.\nSynonyms:\nshine_tree.try_foldl\nshine_tree try_foldl","url":"shine_tree.html#try_foldl"},{"doc":"shine_tree","title":"try_foldr","content":"pub fn try_foldr(\n  tree: ShineTree(a),\n  acc: b,\n  f: fn(b, a) -&gt; Result(b, c),\n) -&gt; Result(b, c)\n Perform a folding operation and return the success `Ok` value,\n or the error `Error` value if the operation fails immediately.\nSynonyms:\nshine_tree.try_foldr\nshine_tree try_foldr","url":"shine_tree.html#try_foldr"},{"doc":"shine_tree","title":"unshift","content":"pub fn unshift(tree: ShineTree(a), value: a) -&gt; ShineTree(a)\n Pushes an element to the beginning of the given tree.\n \n \n let tree = shine_tree.from_list([1, 2, 3])\n shine_tree.unshift(tree, 0)\n // -&gt; shine_tree.from_list([0, 1, 2, 3])\n \nSynonyms:\nshine_tree.unshift\nshine_tree unshift","url":"shine_tree.html#unshift"},{"doc":"shine_tree","title":"shine_tree","content":"","url":"shine_tree.html"}]);